﻿using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.ViewModels;
using SoftwareInstallationDatabaseImplement.Models;
using SoftwareInstallationDataModels.Enums;
using SoftwareInstallationDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace SoftwareInstallationDatabaseImplement
{
    [DataContract]
    public class Order : IOrderModel
    {
        [DataMember]
        public int Id { get; private set; }
        [DataMember]
        [Required]
        public int PackageId { get; private set; }
        [Required]
        [DataMember]
        public int ClientId { get; private set; }
        [DataMember]
        public int? ImplementerId { get; private set; }
        [DataMember]
        [Required]
        public int Count { get; private set; }
        [DataMember]
        [Required]
        public double Sum { get; private set; }
        [DataMember]
        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        [DataMember]
        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        [DataMember]
        public DateTime? DateImplement { get; private set; }

        public virtual Package Package { get; set; }
        public virtual Client Client { get; private set; }
        public virtual Implementer? Implementer { get; set; }

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }

            return new Order()
            {
                Id = model.Id,
                PackageId = model.PackageId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                ClientId = model.ClientId
            };
        }

        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            PackageId = PackageId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            PackageName = Package?.PackageName,
            ClientId = ClientId,
            ClientFIO = Client?.ClientFIO,
            ImplementerFIO = Implementer?.ImplementerFIO ?? string.Empty
        };
    }
}
