﻿using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.ViewModels;
using SoftwareInstallationDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace SoftwareInstallationFileImplement
{
    [DataContract]
    public class MessageInfo : IMessageInfoModel
    {
        [DataMember]
        public string MessageId { get; private set; } = string.Empty;
        [DataMember]
        public int? ClientId { get; private set; }
        [DataMember]
        public string SenderName { get; private set; } = string.Empty;
        [DataMember]
        public DateTime DateDelivery { get; private set; } = DateTime.Now;
        [DataMember]
        public string Subject { get; private set; } = string.Empty;
        [DataMember]
        public string Body { get; private set; } = string.Empty;

        public int Id => throw new NotImplementedException();

        public static MessageInfo? Create(MessageInfoBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new()
            {
                Body = model.Body,
                Subject = model.Subject,
                DateDelivery = model.DateDelivery,
                SenderName = model.SenderName,
                ClientId = model.ClientId,
                MessageId = model.MessageId
            };
        }

        public static MessageInfo? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new()
            {
                Body = element.Attribute("Body")!.Value,
                Subject = element.Attribute("Subject")!.Value,
                DateDelivery = Convert.ToDateTime(element.Attribute("DateDelivery")!.Value),
                SenderName = element.Attribute("SenderName")!.Value,
                ClientId = Convert.ToInt32(element.Attribute("ClientId")!.Value),
                MessageId = element.Attribute("MessageId")!.Value,
            };
        }

        public MessageInfoViewModel GetViewModel => new()
        {
            Body = Body,
            Subject = Subject,
            DateDelivery = DateDelivery,
            SenderName = SenderName,
            ClientId = ClientId,
            MessageId = MessageId
        };

        public XElement GetXElement => new("MessageInfo",
            new XAttribute("Subject", Subject),
            new XAttribute("Body", Body),
            new XAttribute("ClientId", ClientId),
            new XAttribute("MessageId", MessageId),
            new XAttribute("SenderName", SenderName),
            new XAttribute("DateDelivery", DateDelivery)
            );
    }
}
