using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using SoftwareInstallationBusinessLogic.BusinessLogics;
using SoftwareInstallationBusinessLogic.MailWorker;
using SoftwareInstallationBusinessLogic.OfficePackage;
using SoftwareInstallationBusinessLogic.OfficePackage.Implements;
using SoftwareInstallationContracts.BindingModels;
using SoftwareInstallationContracts.BusinessLogicContracts;
using SoftwareInstallationContracts.BusinessLogicsContracts;
using SoftwareInstallationContracts.DI;
using SoftwareInstallationView;

namespace SoftwareInstallation
{
    internal static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            InitDependency();
            try
            {
                var mailSender =
                DependencyManager.Instance.Resolve<AbstractMailWorker>();
                mailSender?.MailConfig(new MailConfigBindingModel
                {
                    MailLogin = System.Configuration.ConfigurationManager.AppSettings["MailLogin"] ?? string.Empty,
                    MailPassword = System.Configuration.ConfigurationManager.AppSettings["MailPassword"] ?? string.Empty,
                    SmtpClientHost = System.Configuration.ConfigurationManager.AppSettings["SmtpClientHost"] ?? string.Empty,
                    SmtpClientPort = Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["SmtpClientPort"]),
                    PopHost = System.Configuration.ConfigurationManager.AppSettings["PopHost"] ?? string.Empty,
                    PopPort = Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["PopPort"])
                });
                //  
                var timer = new System.Threading.Timer(new TimerCallback(MailCheck!), null, 0, 100000);
            }
            catch (Exception ex)
            {
                var logger = DependencyManager.Instance.Resolve<ILogger>();
                logger?.LogError(ex, "   ");
            }
            Application.Run(DependencyManager.Instance.Resolve<FormMain>());
        }
        private static void InitDependency()
        {
            DependencyManager.InitDependency();
            DependencyManager.Instance.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            DependencyManager.Instance.RegisterType<IComponentLogic,
            ComponentLogic>();
            DependencyManager.Instance.RegisterType<IOrderLogic,
            OrderLogic>();
            DependencyManager.Instance.RegisterType<IPackageLogic,
            PackageLogic>();
            DependencyManager.Instance.RegisterType<IReportLogic,
            ReportLogic>();
            DependencyManager.Instance.RegisterType<IClientLogic,
            ClientLogic>();
            DependencyManager.Instance.RegisterType<IImplementerLogic,
            ImplementerLogic>();
            DependencyManager.Instance.RegisterType<IMessageInfoLogic,
            MessageInfoLogic>();
            DependencyManager.Instance.RegisterType<AbstractSaveToExcel,
            SaveToExcel>();
            DependencyManager.Instance.RegisterType<AbstractSaveToWord,
            SaveToWord>();
            DependencyManager.Instance.RegisterType<AbstractSaveToPdf,
            SaveToPdf>();
            DependencyManager.Instance.RegisterType<IWorkProcess,
            WorkModeling>();
            DependencyManager.Instance.RegisterType<AbstractMailWorker,
            MailKitWorker>(true);
            DependencyManager.Instance.RegisterType<IBackUpLogic,
            BackUpLogic>();
            DependencyManager.Instance.RegisterType<FormMain>();
            DependencyManager.Instance.RegisterType<FormComponent>();
            DependencyManager.Instance.RegisterType<FormComponents>();
            DependencyManager.Instance.RegisterType<FormCreateOrder>();
            DependencyManager.Instance.RegisterType<FormPackage>();
            DependencyManager.Instance.RegisterType<FormPackageComponent>();
            DependencyManager.Instance.RegisterType<FormPackages>();
            DependencyManager.Instance.RegisterType<FormReportPackageComponents>();
            DependencyManager.Instance.RegisterType<FormReportOrders>();
            DependencyManager.Instance.RegisterType<FormClients>();
            DependencyManager.Instance.RegisterType<FormImplementer>();
            DependencyManager.Instance.RegisterType<FormImplementers>();
            DependencyManager.Instance.RegisterType<FormMails>();
        }
        private static void MailCheck(object obj) => DependencyManager.Instance.Resolve<AbstractMailWorker>()?.MailCheck();
    }

}